package junit.extensions.eclipse.diff;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.jdt.internal.junit.ui.ITestSelectionListener;
import org.eclipse.jdt.internal.junit.ui.TestRunInfo;
import org.eclipse.jdt.internal.junit.ui.TestRunnerViewPart;
import org.eclipse.jdt.internal.junit.ui.TestRunnerViewPartExtension;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

public class JUnitDiffView extends ViewPart implements IPartListener, ITestSelectionListener {

    private JUnitDiffViewer viewer;
    private TestRunnerViewPart testRunnerViewPart;
    private TestRunnerViewPartExtension testRunnerViewPartExtension;
    private String defaultTitle;
    private JUnitDiff junitDiff;

    public void createPartControl(Composite parent) {
        CompareConfiguration cc = new CompareConfiguration();
		viewer = new JUnitDiffViewer(parent, cc);
        viewer.addToolItems(getToolBarManager());
        viewer.setContentProvider(new JUnitDiffContentProvider(cc));

        IPartService partService = getPartService();
        beginListening(partService.getActivePart());
        partService.addPartListener(this);
        
        defaultTitle = getTitle();
    }

    private IToolBarManager getToolBarManager() {
        return getViewSite().getActionBars().getToolBarManager();
    }

    private IPartService getPartService() {
        return getViewSite().getWorkbenchWindow().getPartService();
    }

    public void setFocus() {
    }
    
    public void dispose() {
        endListening();
        getPartService().removePartListener(this);
        super.dispose();
    }

    public void testChanged(TestRunInfo info) {
        JUnitDiff newJUnitDiff = new JUnitDiff(info);
        if (newJUnitDiff.equals(junitDiff))
            return;
        
        junitDiff = newJUnitDiff;
        viewer.setInput(junitDiff);
        String testName = junitDiff.getTestName();
        if (testName != null)
            setTitle(defaultTitle + " - " + testName);
        else
            setTitle(defaultTitle);
    }

    private void beginListening(IWorkbenchPart part) {
        if (!(part instanceof TestRunnerViewPart))
            return;
        if (part == null || testRunnerViewPart == part)
            return;
        endListening();
        testRunnerViewPart = (TestRunnerViewPart) part;
        testRunnerViewPartExtension = new TestRunnerViewPartExtension(testRunnerViewPart);
        testRunnerViewPartExtension.setTestSelectionListener(this);
    }

    private void endListening() {
        if (testRunnerViewPartExtension != null) {
            testRunnerViewPartExtension.dispose();
            testRunnerViewPartExtension = null;
            testRunnerViewPart = null;
        }
    }

    public void partActivated(IWorkbenchPart part) {
        beginListening(part);
    }

    public void partClosed(IWorkbenchPart part) {
        endListening();
    }

    public void partOpened(IWorkbenchPart part) {
        beginListening(part);
    }

    public void partDeactivated(IWorkbenchPart part) {
    }

    public void partBroughtToTop(IWorkbenchPart part) {
    }
}
